#!/usr/bin/env python3
# coding: utf-8

import argparse
import json
import mimetypes
import os
import subprocess
import sys
import time
from datetime import datetime
from glob import glob
from pathlib import Path
from platform import platform
from typing import Optional

import requests
from rich import print as rprint
from rich.highlighter import JSONHighlighter
from rich.panel import Panel
from rich.progress import track

from .__version__ import __version__


def upload(file: str, best_server: str, folder_id: Optional[str] = None):
    f_obj = Path(file)
    content_type = mimetypes.guess_type(f_obj)[0]
    upload_url = f'https://{best_server}.gofile.io/uploadFile'
    with open(f_obj, 'rb') as f:
        f_data = f.read()

    attempt = 0
    while True:
        try:
            resp = requests.post(
                upload_url,
                data={
                    'token': os.getenv('GOFILE_TOKEN'),
                    'folderId': folder_id
                },
                files={'file': (f_obj.name, f_data, content_type)})
            break
        except requests.exceptions.ConnectionError:
            rprint(
                'The connection was refused from the API side! '
                f'Trying again... ([cyan]{attempt}[/cyan]/10)',
                style='red')
            time.sleep(2)
            attempt += 1
            if attempt > 10:
                break
    return resp


def gofile_upload(path: list,
                  to_single_folder: bool = False,
                  verbose: bool = False,
                  export: bool = False,
                  open_urls: bool = False):
    highlighter = JSONHighlighter()

    get_server = requests.get('https://apiv2.gofile.io/getServer')
    best_server = get_server.json()['data']['server']

    files = []

    for _path in path:
        if not Path(_path).exists():
            rprint(
                f'[red]ERROR: [dim blue]"{Path(_path).absolute()}"[/dim blue] '
                'does not exist! [/red]')
            continue
        if Path(_path).is_dir():
            dir_items = glob(str(Path(f'{_path}/**/*')), recursive=True)
            local_files = [x for x in dir_items if not Path(x).is_dir()]
            files.append(local_files)
        else:
            files.append([_path])

    files = sum(files, [])

    export_data = []
    urls = []
    folder_id = None
    n = 0

    for file in track(files, description='[magenta]Uploading progress:'):
        upload_resp = upload(file, best_server, folder_id).json()

        if to_single_folder and not os.getenv('GOFILE_TOKEN'):
            rprint('[red]ERROR: Gofile token is required when passing '
                   '`--to-single-folder`![/red]\n[dim red]You can find your '
                   'account token on this page: '
                   '[u][blue]https://gofile.io/myProfile[/blue][/u]\nCopy it '
                   'then export it as `GOFILE_TOKEN`. For example:\n'
                   'export GOFILE_TOKEN=\'xxxxxxxxxxxxxxxxx\'[/dim red]')
            sys.exit(1)
        elif to_single_folder and os.getenv('GOFILE_TOKEN'):
            folder_id = upload_resp['data']['parentFolder']

        ts = datetime.now().strftime('%d-%m-%Y %H:%M:%S')
        file_abs = str(Path(file).absolute())
        record = {'file': file_abs, 'timestamp': ts, 'response': upload_resp}

        url = upload_resp['data']['downloadPage']
        urls.append(url)

        if verbose:
            highlighted_resp = highlighter(json.dumps(record, indent=2))
            rprint(Panel(highlighted_resp))

        elif not to_single_folder:
            rprint(
                Panel.fit(
                    f'[yellow]File:[/yellow] [blue]{file}[/blue]\n'
                    f'[yellow]Download page:[/yellow] [u][blue]{url}[/blue][/u]'
                ))
        if export:
            export_data.append(record)

    if not urls:
        sys.exit()

    if to_single_folder:
        files = '\n'.join([str(Path(x).absolute()) for x in files])
        rprint(
            Panel.fit(f'[yellow]Files:[/yellow]\n[blue]{files}[/blue]\n'
                      '[yellow]Download page:[/yellow] '
                      f'[u][blue]{urls[0]}[/blue][/u]'))

    if export:
        export_fname = f'gofile_export_{int(time.time())}.json'
        with open(export_fname, 'w') as j:
            json.dump(export_data, j, indent=4)
        rprint('[green]Exported data to:[/green] '
               f'[magenta]{export_fname}[/magenta]')

    if 'macOS' in platform() and open_urls:
        for url in urls:
            subprocess.call(['open', f'{url}'])
            if to_single_folder:
                break


def opts():
    parser = argparse.ArgumentParser(
        description='Example: gofile <file/folder_path>')
    parser.add_argument(
        '-s',
        '--to-single-folder',
        help=
        'Upload multiple files to the same folder. All files will share the '
        'same URL. This option requires a valid token exported as: '
        '`GOFILE_TOKEN`',
        action='store_true')
    parser.add_argument(
        '-o',
        '--open-urls',
        help='Open the URL(s) in the browser when the upload is complete '
        '(macOS-only)',
        action='store_true')
    parser.add_argument('-e',
                        '--export',
                        help='Export upload response(s) to a JSON file',
                        action='store_true')
    parser.add_argument('-vv',
                        '--verbose',
                        help='Show more information',
                        action='store_true')
    parser.add_argument('path',
                        nargs='+',
                        help='Path to the file(s) and/or folder(s)')
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version=f'%(prog)s {__version__}')
    return parser.parse_args()


def main():
    args = opts()
    gofile_upload(
        path=args.path,
        to_single_folder=args.to_single_folder,
        verbose=args.verbose,
        export=args.export,
        open_urls=args.open_urls,
    )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gofilepy']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.0', 'rich>=9.0']

entry_points = \
{'console_scripts': ['gofile = gofilepy:gofile.main',
                     'gofilepy = gofilepy:gofile.main']}

setup_kwargs = {
    'name': 'gofilepy',
    'version': '0.3.0',
    'description': 'Upload files to Gofile.io',
    'long_description': '# gofile\n\n🚀 Python wrapper for gofile.io Upload API[^1].\n\n[![Build](https://github.com/Alyetama/gofile/actions/workflows/poetry-build.yml/badge.svg)](https://github.com/Alyetama/gofile/actions/workflows/poetry-build.yml) [![PyPI version](https://badge.fury.io/py/gofilepy.svg)](https://pypi.org/project/gofilepy/0.2.1) [![Supported Python versions](https://img.shields.io/badge/Python-%3E=3.6-blue.svg)](https://www.python.org/downloads/) [![PEP8](https://img.shields.io/badge/Code%20style-PEP%208-orange.svg)](https://www.python.org/dev/peps/pep-0008/) \n\n\n## Requirements\n- 🐍 [python>=3.6](https://www.python.org/downloads/)\n\n\n## ⬇️ Installation\n\n```sh\npip install gofilepy\n```\n\n\n## ⌨️ Usage\n\n```\nusage: gofilepy [-h] [-s] [-o] [-e] [-vv] [-v] path [path ...]\n\nExample: gofile <file/folder_path>\n\npositional arguments:\n  path                  Path to the file(s) and/or folder(s)\n\noptions:\n  -h, --help            show this help message and exit\n  -s, --to-single-folder\n                        Upload multiple files to the same folder. All files\n                        will share the same URL. This option requires a valid\n                        token exported as: `GOFILE_TOKEN`\n  -o, --open-urls       Open the URL(s) in the browser when the upload is\n                        complete (macOS-only)\n  -e, --export          Export upload response(s) to a JSON file\n  -vv, --verbose        Show more information\n  -v, --version         show program\'s version number and exit\n```\n\n## 📕 Examples\n\n### Example 1: Uploading one file\n```sh\n➜ gofile foo.txt\n╭───────────────────────────────────────────╮\n│ File: foo.txt                             │\n│ Download page: https://gofile.io/d/PkdZP5 │\n╰───────────────────────────────────────────╯\nUploading progress: ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00\n```\n\n### Example 2: Uploading multiple files/directories\n```bash\n➜ gofile foo.txt bar.txt foobar.txt foo/\n╭───────────────────────────────────────────╮\n│ File: foo.txt                             │\n│ Download page: https://gofile.io/d/rLwQVZ │\n╰───────────────────────────────────────────╯\n╭───────────────────────────────────────────╮\n│ File: bar.txt                             │\n│ Download page: https://gofile.io/d/DdS7mZ │\n╰───────────────────────────────────────────╯\n╭───────────────────────────────────────────╮\n│ File: foobar.txt                          │\n│ Download page: https://gofile.io/d/C1VicP │\n╰───────────────────────────────────────────╯\n╭───────────────────────────────────────────╮\n│ File: foo/foo_1.txt                       │\n│ Download page: https://gofile.io/d/CkYw18 │\n╰───────────────────────────────────────────╯\nUploading progress: ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00\n```\n\n### Example 3: Uploading multiple files to the same URL\n\nThis option requires a gofile token (see: [## Misc.](#misc)).\n\n```bash\n➜ gofile -s foo.txt bar.txt\nUploading progress: ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00\n╭───────────────────────────────────────────╮\n│ Files:                                    │\n│ foo.txt                                   │\n│ bar.txt                                   │\n│ Download page: https://gofile.io/d/bFwawd │\n╰───────────────────────────────────────────╯\n```\n\n### Example 4: Verbose output\n```sh\n➜ gofile -vv foo.txt\n╭──────────────────────────────────────────────────────────────────────────────╮\n│ {                                                                            │\n│   "foo.txt": {                                                               │\n│     "timestamp": "30-05-2022 18:42:15",                                      │\n│     "response": {                                                            │\n│       "status": "ok",                                                        │\n│       "data": {                                                              │\n│         "guestToken": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",                    │\n│         "downloadPage": "https://gofile.io/d/sU4hV1",                        │\n│         "code": "sU4hV1",                                                    │\n│         "parentFolder": "7ad2d249-96a1-4675-b185-05665fbc9a46",              │\n│         "fileId": "0e93e093-d122-4e42-a1be-2e7d34d78ffb",                    │\n│         "fileName": "foo.txt",                                               │\n│         "md5": "d41d8cd98f00b204e9800998ecf8427e"                            │\n│       }                                                                      │\n│     }                                                                        │\n│   }                                                                          │\n│ }                                                                            │\n╰──────────────────────────────────────────────────────────────────────────────╯\nUploading progress: ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00\n```\n\n### Example 5: Exporting the API response to a JSON file\n```sh\n➜ gofile -e foo.txt\n╭───────────────────────────────────────────╮\n│ File: foo.txt                             │\n│ Download page: https://gofile.io/d/8t79Lz │\n╰───────────────────────────────────────────╯\nUploading progress: ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00\nExported data to: gofile_export_1653950555.json\n```\n\n<details>\n  <summary>Content of <code>gofile_export_1653950555.json</code></summary>\n  \n```json\n[\n    {\n        "foo.txt": {\n            "timestamp": "30-05-2022 18:42:35",\n            "response": {\n                "status": "ok",\n                "data": {\n                    "guestToken": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",\n                    "downloadPage": "https://gofile.io/d/8t79Lz",\n                    "code": "8t79Lz",\n                    "parentFolder": "05xd4cb-8965-417f-ae34-a116et99b798",\n                    "fileId": "d1fc3a97-8xe3-486a-bc0d-edb1rb103040",\n                    "fileName": "foo.txt",\n                    "md5": "d41d8cd99f00b204e9810998ecf8427e"\n                }\n            }\n        }\n    }\n]\n```\n</details>\n\n## Misc.\n\n### 🔑 Optional: Saving uploads to your Gofile account\n\nIf you want the files to be uploaded to a specific account, you can export your gofile token, which can be retrieved from the [profile page](https://gofile.io/myProfile), as an environment variable `GOFILE_TOKEN`.\n\n```sh\nexport GOFILE_TOKEN=\'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\'\n```\n\n\n[^1]: **Disclaimer: This tool is not associated with Gofile, WOJTEK SAS ©, or the Gofile Team.**\n',
    'author': 'Mohammad Alyetama',
    'author_email': 'malyetama@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
